# Common setup for helper_functions tests

. ${UCF_TEST_LIBDIR}/ucf_helper_functions.sh

PKGNAME="helper-test-pkg"
PKG_DIR=${DPKG_ROOT}${UCF_TEST_TARGET}/pkg
LOC_DIR=${UCF_TEST_TARGET}/loc
DPKG_ROOT_LOC_DIR=${DPKG_ROOT}${LOC_DIR}

mkdir -p ${PKG_DIR} ${DPKG_ROOT_LOC_DIR}
cp -a input.conf ${PKG_DIR}
